@@ -52,6 +52,7 @@ module Text.Pandoc.Parsing ( (>>~),
                              failUnlessLHS,
                              escaped,
                              characterReference,
+                             updateLastStrPos,
                              anyOrderedListMarker,
                              orderedListMarker,
                              charRef,
@@ -652,7 +653,8 @@ data ParserState = ParserState
       stateExamples        :: M.Map String Int, -- ^ Map from example labels to numbers 
       stateHasChapters     :: Bool,          -- ^ True if \chapter encountered
       stateApplyMacros     :: Bool,          -- ^ Apply LaTeX macros?
-      stateMacros          :: [Macro]        -- ^ List of macros defined so far
+      stateMacros          :: [Macro],       -- ^ List of macros defined so far
+      stateRstDefaultRole  :: String         -- ^ Current rST default interpreted text role
     }
     deriving Show
 
@@ -682,7 +684,8 @@ defaultParserState =
                   stateExamples        = M.empty,
                   stateHasChapters     = False,
                   stateApplyMacros     = True,
-                  stateMacros          = []}
+                  stateMacros          = [],
+                  stateRstDefaultRole  = "title-reference"}
 
 data HeaderType 
     = SingleHeader Char  -- ^ Single line of characters underneath
@@ -784,6 +787,10 @@ charOrRef cs =
                        guard (c `elem` cs)
                        return c)
 
+updateLastStrPos :: GenParser Char ParserState ()
+updateLastStrPos = getPosition >>= \p -> 
+  updateState $ \s -> s{ stateLastStrPos = Just p }
+
 singleQuoteStart :: GenParser Char ParserState ()
 singleQuoteStart = do
   failIfInQuoteContext InSingleQuote
