@@ -121,8 +121,12 @@ cssURLs userdata d orig =
                   let url = toString
                           $ case B.take 1 u of
                                  "\"" -> B.takeWhile (/='"') $ B.drop 1 u
+                                 "'"  -> B.takeWhile (/='\'') $ B.drop 1 u
                                  _    -> u
-                  (raw, mime) <- getRaw userdata "" (d </> url)
+                  let url' = if isAbsoluteURI url
+                                then url
+                                else d </> url
+                  (raw, mime) <- getRaw userdata "" url'
                   rest <- cssURLs userdata d v
                   let enc = "data:" `B.append` fromString mime `B.append`
                                ";base64," `B.append` (encode raw)
@@ -157,6 +161,16 @@ makeSelfContained :: Maybe FilePath -> String -> IO String
 makeSelfContained userdata inp = do
   let tags = parseTags inp
   out' <- mapM (convertTag userdata) tags
-  return $ renderTagsOptions renderOptions{ optMinimize = (\t -> t == "br"
-                 || t == "img" || t == "meta" || t == "link" ) } out'
+  return $ renderTags' out'
 
+-- repeated from HTML reader:
+renderTags' :: [Tag String] -> String
+renderTags' = renderTagsOptions
+               renderOptions{ optMinimize = \x ->
+                                    let y = map toLower x
+                                    in  y == "hr" || y == "br" ||
+                                        y == "img" || y == "meta" ||
+                                        y == "link"
+                            , optRawTag = \x ->
+                                    let y = map toLower x
+                                    in  y == "script" || y == "style" }
