@@ -48,6 +48,8 @@ import Text.Pandoc.Writers.Markdown ( writePlain )
 import Data.Char ( toLower )
 import Network.URI ( unEscapeString )
 import Text.Pandoc.MIME (getMimeType)
+import Prelude hiding (catch)
+import Control.Exception (catch, SomeException)
 
 -- | Produce an EPUB file from a Pandoc document.
 writeEPUB :: Maybe String   -- ^ EPUB stylesheet specified at command line
@@ -126,8 +128,9 @@ writeEPUB mbStylesheet fonts opts doc@(Pandoc meta _) = do
   let chapterEntries = zipWith chapterToEntry [1..] chapters
 
   -- contents.opf
-  localeLang <- catch (liftM (takeWhile (/='.')) $ getEnv "LANG")
-                    (\_ -> return "en-US")
+  localeLang <- catch (liftM (map (\c -> if c == '_' then '-' else c) .
+                       takeWhile (/='.')) $ getEnv "LANG")
+                    (\e -> let _ = (e :: SomeException) in return "en-US")
   let lang = case lookup "lang" (writerVariables opts') of
                      Just x  -> x
                      Nothing -> localeLang
