@@ -40,7 +40,7 @@ import Text.Printf ( printf )
 import Control.Applicative ( (<$>) )
 import Control.Arrow ( (***), (>>>) )
 import Control.Monad.State hiding ( when )
-import Data.Char (chr)
+import Data.Char (chr, isDigit)
 import qualified Data.Map as Map
 
 -- | Auxiliary function to convert Plain block to Para.
@@ -154,8 +154,8 @@ inHeaderTags i d =
                                  , ("text:outline-level", show i)] d
 
 inQuotes :: QuoteType -> Doc -> Doc
-inQuotes SingleQuote s = text "&#8216;" <> s <> text "&#8217;"
-inQuotes DoubleQuote s = text "&#8220;" <> s <> text "&#8221;"
+inQuotes SingleQuote s = char '\8216' <> s <> char '\8217'
+inQuotes DoubleQuote s = char '\8220' <> s <> char '\8221'
 
 handleSpaces :: String -> Doc
 handleSpaces s
@@ -361,10 +361,6 @@ inlinesToOpenDocument o l = hcat <$> mapM (inlineToOpenDocument o) l
 -- | Convert an inline element to OpenDocument.
 inlineToOpenDocument :: WriterOptions -> Inline -> State WriterState Doc
 inlineToOpenDocument o ils
-    | Ellipses      <- ils = inTextStyle $ text "&#8230;"
-    | EmDash        <- ils = inTextStyle $ text "&#8212;"
-    | EnDash        <- ils = inTextStyle $ text "&#8211;"
-    | Apostrophe    <- ils = inTextStyle $ text "&#8217;"
     | Space         <- ils = inTextStyle space
     | LineBreak     <- ils = return $ selfClosingTag "text:line-break" []
     | Str         s <- ils = inTextStyle $ handleSpaces $ escapeStringForXML s
@@ -382,7 +378,7 @@ inlineToOpenDocument o ils
     | RawInline "html" s <- ils = preformatted s  -- for backwards compat.
     | RawInline _ _ <- ils = return empty
     | Link  l (s,t) <- ils = mkLink s t <$> inlinesToOpenDocument o l
-    | Image _ (s,_) <- ils = return $ mkImg  s
+    | Image _ (s,t) <- ils = return $ mkImg  s t
     | Note        l <- ils = mkNote l
     | otherwise            = return empty
     where
@@ -391,7 +387,7 @@ inlineToOpenDocument o ils
                                            , ("xlink:href" , s       )
                                            , ("office:name", t       )
                                            ] . inSpanTags "Definition"
-      mkImg  s     = inTags False "draw:frame" [] $
+      mkImg  s t   = inTags False "draw:frame" (attrsFromTitle t) $
                      selfClosingTag "draw:image" [ ("xlink:href"   , s       )
                                                  , ("xlink:type"   , "simple")
                                                  , ("xlink:show"   , "embed" )
@@ -407,6 +403,17 @@ inlineToOpenDocument o ils
         addNote nn
         return nn
 
+-- a title of the form "120x140" will be interpreted as image
+-- size in points.
+attrsFromTitle :: String -> [(String,String)]
+attrsFromTitle s = if null xs || null ys
+                      then []
+                      else [("svg:width",xs ++ "pt"),("svg:height",ys ++ "pt")]
+  where (xs,rest) = span isDigit s
+        ys        = case rest of
+                         ('x':zs) | all isDigit zs -> zs
+                         _ -> ""
+
 bulletListStyle :: Int -> State WriterState (Int,(Int,[Doc]))
 bulletListStyle l =
     let doStyles  i = inTags True "text:list-level-style-bullet"
